/*esccom fundamental escape probability radiative transfer routine, for complete redistribution */
#include <string.h>
#include "cddefines.h"
#include "escdst.h"
#include "escmase.h"
#include "esa0k2.h"
#include "esccom.h"

double esccom(double tau, 
  double a)
{
	double atau, 
	  b, 
	  esccom_v;

#	ifdef DEBUG_FUN
	fputs( "<+>esccom()\n", debug_fp );
#	endif

	/* this is one of the three fundamental escape probability routines
	 * the three are esccom, escinc, and escla
	 * it computes esc prob for complete redistribution
	 * computes escape prob for complete redistribution in one direction
	 * */
	if( tau < 0. )
	{
		/* line mased */
		esccom_v = escmase(tau);
		
#	ifdef DEBUG_FUN
		fputs( " <->esccom()\n", debug_fp );
#	endif
		return( esccom_v );
	}

	if( strcmp(escdst.chEscFun,"SIMP") == 0 )
	{
		/* this set with "escape simple" command, used for debugging */
		esccom_v = 1./(1. + tau);
		
#	ifdef DEBUG_FUN
		fputs( " <->esccom()\n", debug_fp );
#	endif
		return( esccom_v );
	}

	/* chEscFun is the escape prob function, usually k2
	 * */
	if( strcmp(escdst.chEscFun,"INCO") == 0 )
	{
		if( tau < 10. )
		{
			esccom_v = 1./(1. + 1.6*tau);
			
#	ifdef DEBUG_FUN
			fputs( " <->esccom()\n", debug_fp );
#	endif
			return( esccom_v );
		}

		/* first find coeficient b(tau) */
		atau = a*tau;
		if( atau > 1. )
		{
			b = 1.6 + (3.*pow(2.*a,-0.12))/(1. + atau);
		}
		else
		{
			b = 1.6 + (3.*pow(2.*a,-0.12))/(1. + 1./sqrt(atau));
		}
		b = MIN2(6.,b);

		esccom_v = 1./(1. + b*tau) + 0.25*sqrt(a/tau);

	}
	else if( strcmp(escdst.chEscFun,"  K2") == 0 )
	{
		/* this is the usual case, Hummer's k2 function
		 * error! what happens if negative tau? */
		esccom_v = esa0k2(tau);
	}
	else
	{
		fprintf( ioQQQ, " Unknown escape probability function=%4.4s\n", 
		  escdst.chEscFun );
		CloseUnits();
		puts( "[Stop in esccom]" );
		exit(1);
	}


#	ifdef DEBUG_FUN
	fputs( " <->esccom()\n", debug_fp );
#	endif
	return( esccom_v );
}

