/*escgrd escape probability radiative transfer for incomplete redistribution */
#include "cddefines.h"
#define	SCALE	2.
#include "touton.h"
#include "twoway.h"
#include "showme.h"
#include "escinc.h"
#include "escgrd.h"

double escgrd(double tau, 
  double tout, 
  double damp )
{
	double escgrd_v, 
	  tt;

#	ifdef DEBUG_FUN
	fputs( "<+>escgrd()\n", debug_fp );
#	endif

	/* find escape prob for incomp redis, average of two 1-sided probs*/

	if( touton.lgTauOutOn )
	{
		/*  outward optical depth if defined */
		tt = tout - tau;
		/*  help convergence by not letting tau go to zero at back edge of
		 *  when there was a bad guess for the total optical depth
		 *  note that this test is seldom hit since makestatrt does check
		 *  for overrun */
		if( tt < 0. )
			tt = (float)(tau/SCALE);
		twoway.wayin = (float)escinc(tau,damp);
		twoway.wayout = (float)escinc(tt,damp);
		twoway.fracin = twoway.wayin/(twoway.wayin + twoway.wayout);
		escgrd_v = 0.5*(twoway.wayin + twoway.wayout);
	}
	else
	{
		/*  outward optical depth not defined, dont estimate fraction out */
		twoway.fracin = 0.;
		twoway.wayout = 1.;
		escgrd_v = escinc(tau,damp);
		twoway.wayin = (float)escgrd_v;
	}

	/*begin sanity check */
	if( escgrd_v <= 0. )
	{
		fprintf( ioQQQ, " escgrd finds insane escape probability,%10.2e\n", 
		  escgrd_v );
		fprintf( ioQQQ, " tau, tout, damp=%10.2e%10.2e%10.2e\n", 
		  tau, tout, damp );
		ShowMe();
		CloseUnits();
		puts( "[Stop in escgrd]" );
		exit(1);
	}
	/*end sanity check
	 * */

#	ifdef DEBUG_FUN
	fputs( " <->escgrd()\n", debug_fp );
#	endif
	return( escgrd_v );
}

