/*escinc fundamental escape probability radiative transfer routine for incomplete redistribution */
#include <string.h>
#include "cddefines.h"
#include "escdst.h"
#include "escmase.h"
#include "escinc.h"

double escinc(double tau, 
  double a)
{
	double atau, 
	  b, 
	  escinc_v;

#	ifdef DEBUG_FUN
	fputs( "<+>escinc()\n", debug_fp );
#	endif


	/* this is one of the three fundamental escape probability routines
	 * the three are esccom, escinc, and escla
	 * it computes esc prob for incomplete redistribution
	 * */
	if( strcmp(escdst.chEscFun,"SIMP") == 0 )
	{
		/* this set with "escape simple" command, used for debugging */
		escinc_v = 1./(1. + tau);
		
#	ifdef DEBUG_FUN
		fputs( " <->escinc()\n", debug_fp );
#	endif
		return( escinc_v );
	}

	if( tau < 0. )
	{
		/* line mased */
		escinc_v = escmase(tau);
	}
	else if( tau < 10. )
	{
		/* linear part of doppler core */
		escinc_v = 1./(1. + 1.6*tau);
	}
	else
	{
		/* first find coeficient b(tau) */
		atau = a*tau;
		if( atau > 1. )
		{
			b = 1.6 + (3.*pow(2.*a,-0.12))/(1. + atau);
		}
		else
		{
			b = 1.6 + (3.*pow(2.*a,-0.12))/(1. + 1./sqrt(atau));
		}
		b = MIN2(6.,b);

		escinc_v = 1./(1. + b*tau);
	}


#	ifdef DEBUG_FUN
	fputs( " <->escinc()\n", debug_fp );
#	endif
	return( escinc_v );
}

