/*escla escape prob for hydrogen atom Lya, using Hummer and Kunasz results,
 * called by hydropesc */
#include <string.h>
#include "cddefines.h"
#include "physconst.h"
#include "doppvel.h"
#include "opac.h"
#include "redis.h"
#include "twoway.h"
#include "humla.h"
#include "escsub.h"
#include "taulines.h"
#include "escla.h"

double escla(float *esin, 
  float *dest, 
  double abund, 
  long int ipZ)/* 0 for H, */
{
	double beta, 
	  conopc, 
	  damp, 
	  escla_v;
	 float dstin, 
	  dstout;

#	ifdef DEBUG_FUN
	fputs( "<+>escla()\n", debug_fp );
#	endif

	/* 
	 * this is one of the three fundamental escape probability functions
	 * the three are esccom, escinc, and escla
	 * evaluate esc prob for LA
	 * optical depth in outer direction always defined
	 */

	/* check charge */
	assert( ipZ >= 0 );
	assert( ipZ < LIMELM );

	if( HydroLines[ipZ][IP2P][IP1S][IPLNTAUTOT-1] - 
	  HydroLines[ipZ][IP2P][IP1S][IPLNTAUIN-1] < 0. )
	{
		/* this is the case if we overrun the optical depth scale
		 * just leave things as they are */
		escla_v = HydroLines[ipZ][IP2P][IP1S][IPLNESCP-1];
		twoway.fracin = HydroLines[ipZ][IP2P][IP1S][IPLNINWD-1];
		*esin = twoway.fracin;
		*dest = HydroLines[ipZ][IP2P][IP1S][IPLNDESP-1];
		
#		ifdef DEBUG_FUN
		fputs( " <->escla()\n", debug_fp );
#		endif
		return( escla_v );
	}

	if( strcmp(redis.chRedis,"INCO") == 0 )
	{
		/* incomplete redistribution */
		conopc = opac.opac[(long)(HydroLines[ipZ][IP2P][IP1S][IPLNIPCONT-1])-1];
		if( abund > 0. )
		{
			beta = conopc/(abund/SQRTPI*HydroLines[ipZ][IP2P][IP1S][IPLNOPAC-1]/
			  DoppVel.doppler[ipZ] + conopc);
		}
		else
		{
			beta = 1.;
		}

		/* WAYIN will be escape prob in inward direction */
		humla(HydroLines[ipZ][IP2P][IP1S][IPLNTAUIN-1],
		  beta,&twoway.wayin,&dstin , 
		  (long)(HydroLines[ipZ][IP2P][IP1S][IPLNIPCONT-1])-1);

		if( ((twoway.wayin > 1. || twoway.wayin < 0.) || dstin > 1.) || 
		  dstin < 0. )
		{
			fprintf( ioQQQ, 
				" HUMLA returns inwd prob out of bounds;  esc=%10.2e dst="
				"%10.2e beta=%10.2e tau=%10.2e\n", 
			  twoway.wayin, dstin, beta, HydroLines[ipZ][IP2P][IP1S][IPLNTAUIN-1] );
			CloseUnits();
			puts( "[Stop in escla]" );
			exit(1);
		}

		humla(MAX2(HydroLines[ipZ][IP2P][IP1S][IPLNTAUTOT-1]/
		  100.,HydroLines[ipZ][IP2P][IP1S][IPLNTAUTOT-1]-
		  HydroLines[ipZ][IP2P][IP1S][IPLNTAUIN-1]),
		  beta,&twoway.wayout,&dstout, 
		  (long)(HydroLines[ipZ][IP2P][IP1S][IPLNIPCONT-1])-1);

		if( ((twoway.wayout > 1. || twoway.wayout < 0.) || dstout > 1.) || 
			dstout < 0. )
		{
			fprintf( ioQQQ, 
				" HUMLA returns outwd prob out of bounds;  esc=%10.2e dst=%10.2e beta=%10.2e tau=%10.2e\n", 
			  twoway.wayout, dstout, beta, 
			  MAX2(HydroLines[ipZ][IP2P][IP1S][IPLNTAUTOT-1]/
			  100.,HydroLines[ipZ][IP2P][IP1S][IPLNTAUTOT-1]-
			  HydroLines[ipZ][IP2P][IP1S][IPLNTAUIN-1]) );
			CloseUnits();
			puts( "[Stop in escla]" );
			exit(1);
		}

		escla_v = (twoway.wayin + twoway.wayout)/2.;
		*esin = twoway.wayin;
		*dest = (dstin + dstout)/2.f;
		twoway.fracin = twoway.wayin/(twoway.wayin + twoway.wayout);
	}

	else if( (strcmp(redis.chRedis,"COMP") == 0 ) ||
		(strcmp(redis.chRedis,"WIND") == 0 ) )
	{
		/* wind or complete redis */
		/*>>>chng 98 dec 18, set damp to zero before call, had not
		 * been initialized */
		damp = 0.;
		escla_v = escsub(HydroLines[ipZ][IP2P][IP1S][IPLNTAUIN-1],
		  HydroLines[ipZ][IP2P][IP1S][IPLNTAUTOT-1],
		  damp);
		*esin = twoway.wayin;
		twoway.fracin = 0.5;
		*dest = 0.;
		/* not used, but keep lint quiet */
		beta = 1.;
	}

	else
	{
		/* not used, but keep lint quiet */
		beta = 1.;
		fprintf( ioQQQ,"escla called with insane redistribution function ==%s==\n",
			redis.chRedis );
		CloseUnits();
		puts( "[Stop in escla]" );
		exit(1);
	}

	assert( escla_v >= 0. );

	/* this is for debugging H Lya */
	{
		enum {BUG=FALSE};
		if( BUG )
		{
			if( ipZ == 0 )
			{
				fprintf(ioQQQ," %.2e %.2e %.2e %.2e %.2e \n",
				HydroLines[ipZ][IP2P][IP1S][IPLNTAUIN-1],
				beta, 
				opac.albedo[(long)(HydroLines[ipZ][IP2P][IP1S][IPLNIPCONT-1])-1] ,
				escla_v , 
				*dest 
				);
			}
		}
	}

#	ifdef DEBUG_FUN
	fputs( " <->escla()\n", debug_fp );
#	endif
	return( escla_v );
}

