/*escmase escape probability for negative (masing) optical depths */
#include "cddefines.h"
#include "escmase.h"
#include "findneg.h"

double escmase(double tau)
{
	double a, 
	  escmase_v;
	static float zero = 0.;

#	ifdef DEBUG_FUN
	fputs( "<+>escmase()\n", debug_fp );
#	endif

	/* this is the only routine that computes maser escape proba
	 *begin sanity check */
	if( tau > 0. )
	{
		fprintf( ioQQQ, " escmase called with positive tau!!\n" );
		fprintf( ioQQQ, " Tau =%10.2e\n", tau );
		CloseUnits();
		puts( "[Stop in escmase]" );
		exit(1);
	}
	/*end sanity check */
	if( tau > -0.1 )
	{
		escmase_v = 1. - tau*(0.5 + tau/6.);
	}
	else if( tau > -30. )
	{
		escmase_v = (1. - exp(-tau))/tau;
	}
	else
	{
		fprintf( ioQQQ, " escmase called with 2big tau%10.2e\n", 
		  tau );
		FindNeg();
		fprintf( ioQQQ, " will now div by 0 to get trace\n" );
		a = 1./zero;
		fprintf( ioQQQ, "%10.2e\n", a );
		CloseUnits();
		puts( "[Stop in escmase]" );
		exit(1);
	}

	/*begin sanity check */
	if( escmase_v < 1.0 )
	{
		fprintf( ioQQQ, " escmase finds P<1!!\n" );
		fprintf( ioQQQ, " escmase, Tau =%10.2e%10.2e\n", escmase_v, 
		  tau );
		CloseUnits();
		puts( "[Stop in escmase]" );
		exit(1);
	}
	/*end sanity check
	 * */

#	ifdef DEBUG_FUN
	fputs( " <->escmase()\n", debug_fp );
#	endif
	return( escmase_v );
}

