/*IonSodiu ionization balance for sodium */
#include "cddefines.h"
#	ifdef NDIM
#undef NDIM
#	endif
#	ifdef NELEM
#undef NELEM
#	endif
#define	NDIM	12
#define	NELEM	11
#include "trace.h"
#include "elmton.h"
#include "timed.h"
#include "ionfracs.h"
#include "collidionize.h"
#include "ionzer.h"
#include "makerecomb.h"
#include "theavy.h"
#include "photoionize.h"
#include "bidiag.h"
#include "ionheavy.h"

void IonSodiu()
{
	long int i, 
	  _r;
	static double dicoef[2][NDIM - 1], 
	  dite[2][NDIM - 1];
	static double rec[NDIM - 2]={3.42e-10,1.17e-9,2.60e-9,4.56e-9,6.81e-9,
	  1.07e-8,1.32e-8,1.92e-8,2.24e-8,2.54e-8};
	static double pl[NDIM - 2]={-0.85,-0.73,-0.70,-0.68,-0.66,-0.66,
	  -0.65,-0.68,-0.67,-0.63};
	static double tlow[2]={1.35e-10,-0.682};
	static double ditcrt[NDIM - 1]={0.,1.3e5,4.4e5,7.2e4,2.8e5,4.0e5,
	  1.3e6,1.6e6,1.9e6,3.4e4,0.};
	static double aa[NDIM - 1]={0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.};
	static double bb[NDIM - 1]={0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.};
	static double cc[NDIM - 1]={0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.};
	static double dd[NDIM - 1]={0.,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.};
	static double ff[NDIM - 1]={0.1,0.,0.,0.,0.,0.,0.,0.,0.,0.,0.};
	static int _aini = 1;
	if( _aini ){ /* Do 1 TIME INITIALIZATIONS! */
		{ static double _itmp2[] = {1.0e-3,2.6e-3,6.0e-3,1.1e-2,8.0e-3,
		  1.0e-2,3.2e-2,1.2e-2,2.2e-1,1.8e-1,0.};
		for( i=1, _r = 0; i <= (NDIM - 1); i++ )
		{
			dicoef[0][i-1] = _itmp2[_r++];
			}
		}
		{ static double _itmp3[] = {3.6e5,3.8e5,3.4e5,3.0e5,2.7e5,2.8e5,
		  3.2e5,1.9e5,1.2e7,1.3e7,0.};
		for( i=1, _r = 0; i <= (NDIM - 1); i++ )
		{
			dite[0][i-1] = _itmp3[_r++];
			}
		}
		for( i=1; i <= (NDIM - 1); i++ )
		{
			dicoef[1][i-1] = 0.;
			}
		for( i=1; i <= (NDIM - 1); i++ )
		{
			dite[1][i-1] = 0.;
			}
		_aini = 0;
	}

#	ifdef DEBUG_FUN
	fputs( "<+>IonSodiu()\n", debug_fp );
#	endif

	/* sodium nelem=11
	 *
	 * rates taken from Mg Shull and van Steenberg, Ap.J. Sup 48, 95. */

	/* from Aldrovandi and Pequignot Revista Brasileira de Fisica, 4, 491, */
	/* Pequignot and Aldrovandi Ast Ap 161, 169. */

	/* sodium, atomic number 11 */
	if( !elmton.lgElmtOn[10] )
	{
		
#		ifdef DEBUG_FUN
		fputs( " <->IonSodiu()\n", debug_fp );
#		endif
		return;
	}

	ionzer(NELEM);

	PhotoIonize(NELEM-1,FALSE);

	/* find collisional ionization rates */
	CollidIonize(NELEM-1);

	/* get recombination coefficients */
	MakeRecomb(rec,pl,(double*)dicoef,(double*)dite,ditcrt,aa,bb,cc,
	  dd,ff,NELEM-1,tlow);

	if( timed.itime == -1 )
	{
		theavy(NELEM);
		
#		ifdef DEBUG_FUN
		fputs( " <->IonSodiu()\n", debug_fp );
#		endif
		return;
	}

	/* solve for ionization balance */
	BiDiag(NELEM-1,FALSE);

	if( trace.lgTrace && trace.lgHeavyBug )
	{
		fprintf( ioQQQ, "     IonSodiu returns; frac=" );
		for( i=1; i <= 10; i++ )
		{
			fprintf( ioQQQ, "%10.3e", xIonFracs[i][NELEM-1]/
			  xIonFracs[0][NELEM-1] );
		}
		fprintf( ioQQQ, "\n" );
	}

#	ifdef DEBUG_FUN
	fputs( " <->IonSodiu()\n", debug_fp );
#	endif
	return;
}

