/*level2 do level population and cooling for two level atom */
#include <string.h>
#include "cddefines.h"
#include "taulines.h"
#include "dcool.h"
#include "ionfracs.h"
#include "phycon.h"
#include "fslev.h"
#include "edsqte.h"
#include "poplevls.h"
#include "rfield.h"
#include "chionlbl.h"
#include "sexp.h"
#include "addotslin.h"
#include "coladd.h"
#include "level2.h"

void level2(float tarray[])
{
	char chLab[5];
	long int ion, 
	  ipCont, 
	  nelem;
	float AbunxIon, 
	  Enr12, 
	  Enr21, 
	  a21, 
	  aem, 
	  boltz, 
	  col12, 
	  col21, 
	  coolng, 
	  g1, 
	  g2, 
	  omega, 
	  plower, 
	  r, 
	  rate12, 
	  ri21;

#	ifdef DEBUG_FUN
	fputs( "<+>level2()\n", debug_fp );
#	endif

	/* result is density (cm-3) of excited state times A21
	 * result normalized to N1+N2=ABUND
	 * routine increments dCooldT, call coladd
	 * CDSQTE is EDEN / SQRTE * 8.629E-6
	 */

	ion = (long)(tarray[IPLNIONSTG-1] + 0.00001);
	nelem = (long)(tarray[IPLNNELEM-1] + 0.00001);

	/* xIonFracs(nelem,i) is density of ith ionization stage (cm^-3) */
	AbunxIon = xIonFracs[ion][nelem-1];

	/* continuum pointer */
	ipCont = (long)(tarray[IPLNIPCONT-1]);

	/* approximate boltzmann factor to see if results zero */
	boltz = rfield.ContBoltz[ipCont-1];

	/* collision strength for this transition, omega is zero for hydrogenic
	 * species which are done in special hydro routines */
	omega = tarray[IPLNCS-1];

	/* ROUGH check whether upper level has significant population,*/
	r = (boltz*edsqte.cdsqte + tarray[IPLNPUMP-1])/(edsqte.cdsqte + 
	  tarray[IPLNAUL-1]);

	/* following first test needed for 32 bit cpu on search phase
	 * >>chng 96 july 2, was 1e-30 crashed on dec, change to 1e-25
	 * if( AbunxIon.lt.1e-25 .or. boltz.gt.30. ) then
	 * >>chng 96 july 11, to below, since can be strong pumping when
	 * boltzman factor essentially zero */
	/* omega in following is zero for hydrogenic species, since done
	 * in hydro routines, so this should cause us to quit on this test */
	if( omega*AbunxIon < 1e-25 || r < 1e-25 )
	{
		/* put in pop since possible just too cool */
		tarray[IPLNPOPL-1] = AbunxIon;
		tarray[IPLNPOPOPC-1] = AbunxIon;
		tarray[IPLNPOPU-1] = 0.;
		tarray[IPLNINTEN-1] = 0.;
		tarray[IPLNCOOL-1] = 0.;
		tarray[IPLNNPHOTS-1] = 0.;
		tarray[IPLNOTS-1] = 0.;
		tarray[IPLNAOVTOT-1] = 1.;
		tarray[IPLNCOLOVTOT-1] = 0.;
		tarray[IPLNHEAT-1] = 0.;
		/* level populations */
		PopLevls.PopLevels[0] = AbunxIon;
		PopLevls.PopLevels[1] = 0.;
		PopLevls.DepLTELevels[0] = 1.;
		PopLevls.DepLTELevels[1] = 0.;
		/* add zero to cooling */
		/* >>chng 99 feb 27, no longer dump into array, since no need to */
		/*coladd(chLab,(long)(tarray[IPLNWLANG-1]),0.);*/
		
#		ifdef DEBUG_FUN
		fputs( " <->level2()\n", debug_fp );
#		endif
		return;
	}

	/* net rate down A21*(escape + destruction) */
	a21 = tarray[IPLNAUL-1]*(tarray[IPLNESCP-1] + tarray[IPLNDESP-1]);

	/* put null terminated line label into chLab using line array*/
	chIonLbl(chLab,tarray);

	/* statistical weights of upper and lower levels */
	g1 = tarray[IPLNGL-1];
	g2 = tarray[IPLNGU-1];

	/* now get real Boltzmann factor */
	boltz = tarray[IPLNBOLT-1]/phycon.te;

	assert( boltz > 0. );
	boltz = (float)sexp(boltz);

	assert( g1 > 0. && g2 > 0. );

	col21 = edsqte.cdsqte*omega;

	/* rate 1 to 2 is both collisions and pumping */
	col12 = col21/g1*boltz;
	col21 /= g2;

	/* the total excitation rate from lower to upper, collisional and pumping */
	rate12 = col12 + tarray[IPLNPUMP-1];

	/* induced emissions down */
	ri21 = tarray[IPLNPUMP-1]*g1/g2;

	/* this is the ratio of lower to upper level populations */
	r = (a21 + col21 + ri21)/rate12;

	/* upper level pop */
	fslev.pfs2 = (float)(AbunxIon/(r + 1.));
	PopLevls.PopLevels[1] = fslev.pfs2;
	tarray[IPLNPOPU-1] = fslev.pfs2;

	/* pop of ground */
	fslev.pfs1 = fslev.pfs2*r;
	PopLevls.PopLevels[0] = fslev.pfs1;

	/* departure coef of excited state rel to ground */
	PopLevls.DepLTELevels[0] = 1.;
	if( boltz > 1e-20 && PopLevls.PopLevels[1] > 1e-20 )
	{
		/* this line could have zero boltz factor but radiatively excited
		 * dec alpha does not obey () in fast mode?? */
		PopLevls.DepLTELevels[1] = (PopLevls.PopLevels[1]/PopLevls.PopLevels[0])/
		  (boltz*g2/g1);
	}
	else
	{
		PopLevls.DepLTELevels[1] = 0.;
	}

	/* number of escaping line photons, used elsewhere for outward beam */
	tarray[IPLNNPHOTS-1] = tarray[IPLNAUL-1]*tarray[IPLNESCP-1]*fslev.pfs2;

	/* intensity of line */
	tarray[IPLNINTEN-1] = tarray[IPLNNPHOTS-1]*tarray[IPLNENRERG-1];
	plower = AbunxIon - fslev.pfs2;

	/* ratio of collisional to total excitation */
	tarray[IPLNCOLOVTOT-1] = col12/rate12;

	/* two cases - collisionally excited (usual case) or 
	 * radiatively excited - in which case line can be a heat source
	 * following are correct heat exchange, if will mix to get correct deriv */
	Enr12 = plower*col12*tarray[IPLNENRERG-1];
	Enr21 = fslev.pfs2*col21*tarray[IPLNENRERG-1];

	/* energy exchange due to this level
	 * net cooling due to excit minus part of de-excit */
	tarray[IPLNCOOL-1] = Enr12 - Enr21*tarray[IPLNCOLOVTOT-1];

	/* net heating is remainder */
	tarray[IPLNHEAT-1] = (float)(Enr21*(1. - tarray[IPLNCOLOVTOT-1]));
	coolng = tarray[IPLNCOOL-1];

	/* expression pre jul 3 95, changed for case where line heating dominates
	 * coolng = (plower*col12 - pfs2*col21)*tarray(ipLnEnrErg)
	 * tarray(ipLnCool) = coolng */

	/* add to cooling - heating part was taken out above,
	 * and is not added in here - it will be added to heating(1,23)
	 * in CoolSum */
	coladd(chLab,(long)(tarray[IPLNWLANG-1]),coolng);

	/* derivative of cooling function */
	dcool.dCooldT += coolng*(tarray[IPLNBOLT-1]*dcool.tsq1 - dcool.halfte);

	/* destroyed part of line */
	tarray[IPLNOTS-1] = fslev.pfs2*tarray[IPLNAUL-1]*tarray[IPLNDESP-1];

	if( tarray[IPLNOTS-1] < 0. )
	{
		fprintf( ioQQQ, " level2 finds negative ots rate, rate and wl are%10.2e%7.0f\n", 
		  tarray[IPLNOTS-1], tarray[4] );
		fprintf( ioQQQ, "%10.2e%10.2e%10.2e%10.2e%10.2e%10.2e%10.2e\n", 
		  tarray[16], a21, col21, col12, fslev.pfs2, plower, coolng );
		fprintf( ioQQQ, "%10.2e%10.2e%10.2e%10.2e\n", tarray[0], 
		  tarray[10], tarray[12], tarray[17] );
		CloseUnits();
		puts( "[Stop in level2]" );
		exit(1);
	}

	/*  now add this line to ots field */
	AddOTSLin( tarray[IPLNOTS-1] , ipCont);
	 
	/* compute ratio Aul/(Aul+Cul) */
	aem = tarray[IPLNAUL-1]*tarray[IPLNESCP-1];
	tarray[IPLNAOVTOT-1] = aem/(aem + col21 + ri21);

	/* level population with correction for stimulated emission */
	tarray[IPLNPOPL-1] = PopLevls.PopLevels[0];
	tarray[IPLNPOPOPC-1] = 
		PopLevls.PopLevels[0] - PopLevls.PopLevels[1]*g1/g2;


#	ifdef DEBUG_FUN
	fputs( " <->level2()\n", debug_fp );
#	endif
	return;
}

