/*level3 compute three level atom, 10, 21, and 20 are line */
#include <string.h>
#include "cddefines.h"
#include "showme.h"
#include "ionfracs.h"
#include "edsqte.h"
#include "teinv.h"
#include "phycon.h"
#include "dcool.h"
#include "poplevls.h"
#include "lev3fail.h"
#include "chionlbl.h"
#include "insane.h"
#include "addotslin.h"
#include "coladd.h"
#include "level3.h"
#include "taulines.h"
#include "chlinelbl.h"

void level3(float t10[], 
  float t21[], 
  float t20[])
{
	char chLab[5], 
	  chLab10[11];
	double AbunxIon, 
	  a, 
	  a10, 
	  a20, 
	  a21, 
	  b, 
	  beta, 
	  bolt01, 
	  bolt02, 
	  bolt12, 
	  c, 
	  ener10, 
	  ener20, 
	  ener21, 
	  g0, 
	  g010, 
	  g020, 
	  g1, 
	  g110, 
	  g121, 
	  g2, 
	  g220, 
	  g221, 
	  o10, 
	  o20, 
	  o21, 
	  p0, 
	  p1, 
	  p2, 
	  pump01, 
	  pump02, 
	  pump12;

	double TotCool, 
	  TotHeat, 
	  TotInten, 
	  alpha, 
	  alpha1, 
	  alpha2, 
	  c01, 
	  c02, 
	  c10, 
	  c12, 
	  c20, 
	  c21, 
	  cnet01, 
	  cnet02, 
	  cnet12, 
	  cool01, 
	  cool02, 
	  cool12, 
	  heat10, 
	  heat20, 
	  heat21, 
	  hnet01, 
	  hnet02, 
	  hnet12, 
	  pump10, 
	  pump20, 
	  pump21, 
	  r01, 
	  r02, 
	  r10, 
	  r12, 
	  r20, 
	  r21, 
	  temp01, 
	  temp02, 
	  temp12;

#	ifdef DEBUG_FUN
	fputs( "<+>level3()\n", debug_fp );
#	endif

	/* compute three level atom, 10, 21, and 20 are line
	 * arrays for 10, 21, and 20 transitions.
	 * one can be a dummy */
	/* >>chng 96 dec 6, to double precision due to round off problems below */

	/* generalized three level atom for any ion
	 * sum of three levels normalized to total abundance
	 *
	 * stat weights of all three lines
	 * sanity check will confirm ok */
	g010 = t10[IPLNGL-1];
	g110 = t10[IPLNGU-1];

	g121 = t21[IPLNGL-1];
	g221 = t21[IPLNGU-1];

	g020 = t20[IPLNGL-1];
	g220 = t20[IPLNGU-1];

	/* these are statistical weights */
	if( g010 > 0. )
	{
		g0 = g010;
	}

	else if( g020 > 0. )
	{
		g0 = g020;
	}

	else
	{
		strcpy( chLab10, chLineLbl(t10) );
		fprintf( ioQQQ, " level3: insane stat weights g0 :%10.2e%10.2e %10.10s\n", 
		  g010, g020, chLab10 );
		insane();
		ShowMe();
		exit(1);
	}

	if( g110 > 0. )
	{
		g1 = g110;
	}

	else if( g121 > 0. )
	{
		g1 = g121;
	}

	else
	{
		strcpy( chLab10, chLineLbl(t10) );
		fprintf( ioQQQ, " level3: insane stat weights g1 :%10.2e%10.2e %10.10s\n", 
		  g010, g020, chLab );
		insane();
		ShowMe();
		CloseUnits();
		puts( "[Stop in level3]" );
		exit(1);
	}

	if( g220 > 0. )
	{
		g2 = g220;
	}

	else if( g221 > 0. )
	{
		g2 = g221;
	}

	else
	{
		strcpy( chLab10, chLineLbl(t20) );
		fprintf( ioQQQ, " level3: insane stat weights g2 :%10.2e%10.2e %10.10s\n", 
		  g010, g020, chLab10 );
		insane();
		ShowMe();
		CloseUnits();
		puts( "[Stop in level3]" );
		exit(1);
	}

	/* abundances from the elements grid
	 * one of these must be a true line */
	if( g010 > 0. )
	{
	/* put null terminated line label into chLab using line array*/
		chIonLbl(chLab, t10);
		AbunxIon = xIonFracs[(long)(t10[IPLNIONSTG-1]+0.1)][(long)(t10[IPLNNELEM-1]+
		  0.1)-1];
	}

	else if( g121 > 0. )
	{
	/* put null terminated line label into chLab using line array*/
		chIonLbl(chLab, t21);
		AbunxIon = xIonFracs[(long)(t21[IPLNIONSTG-1]+0.1)][(long)(t21[IPLNNELEM-1]+
		  0.1)-1];
	}

	else
		/* this cannot possibly happen */
	{
		fprintf( ioQQQ, " level3: insanity at g010 g121 branch \n" );
		insane();
		ShowMe();
		CloseUnits();
		puts( "[Stop in level3]" );
		exit(1);
	}

	a = t10[IPLNBOLT-1]*teinvCom.teinv;
	b = t21[IPLNBOLT-1]*teinvCom.teinv;
	c = t20[IPLNBOLT-1]*teinvCom.teinv;

	if( c == 0. )
	{
		c = a + b;
	}

	/* if still neg at end, then success!, in common so possible to
	 * to check why zero returned */
	Lev3Fail.nLev3Fail = -1;
	if( AbunxIon <= 1e-30 || c > 60. )
	{
		Lev3Fail.nLev3Fail = 0;

		/* all populations are zero */
		PopLevls.PopLevels[0] = (float)AbunxIon;
		PopLevls.PopLevels[1] = 0.;
		PopLevls.PopLevels[2] = 0.;

		/* 666 error! these pops ARE NOT defined below */
		PopLevls.DepLTELevels[0] = 1.;
		PopLevls.DepLTELevels[1] = 0.;
		PopLevls.DepLTELevels[2] = 0.;

		/* level populations */
		t21[IPLNPOPOPC-1] = 0.;
		t10[IPLNPOPOPC-1] = (float)AbunxIon;
		t20[IPLNPOPOPC-1] = (float)AbunxIon;
		t21[IPLNPOPL-1] = 0.;
		t10[IPLNPOPL-1] = (float)AbunxIon;
		t20[IPLNPOPL-1] = (float)AbunxIon;
		t21[IPLNPOPU-1] = 0.;
		t10[IPLNPOPU-1] = 0.;
		t20[IPLNPOPU-1] = 0.;

		/* line heating */
		t20[IPLNHEAT-1] = 0.;
		t21[IPLNHEAT-1] = 0.;
		t10[IPLNHEAT-1] = 0.;

		/* intensity of line */
		t21[IPLNINTEN-1] = 0.;
		t10[IPLNINTEN-1] = 0.;
		t20[IPLNINTEN-1] = 0.;

		/* line cooling */
		t20[IPLNCOOL-1] = 0.;
		t21[IPLNCOOL-1] = 0.;
		t10[IPLNCOOL-1] = 0.;

		/* local ots rates */
		t20[IPLNOTS-1] = 0.;
		t21[IPLNOTS-1] = 0.;
		t10[IPLNOTS-1] = 0.;

		/* number of photons in line zero */
		t21[IPLNNPHOTS-1] = 0.;
		t10[IPLNNPHOTS-1] = 0.;
		t20[IPLNNPHOTS-1] = 0.;

		/* fraction that produced lines zero */
		t21[IPLNAOVTOT-1] = 1.;
		t10[IPLNAOVTOT-1] = 1.;
		t20[IPLNAOVTOT-1] = 1.;

		/* ratio coll over total excitation */
		t21[IPLNCOLOVTOT-1] = 0.;
		t10[IPLNCOLOVTOT-1] = 0.;
		t20[IPLNCOLOVTOT-1] = 0.;

		/* add zero to cooling */
		coladd(chLab,(long)(t21[IPLNWLANG-1]),0.);
		coladd(chLab,(long)(t10[IPLNWLANG-1]),0.);
		coladd(chLab,(long)(t20[IPLNWLANG-1]),0.);

#		ifdef DEBUG_FUN
		fputs( " <->level3()\n", debug_fp );
#		endif
		return;
	}

	/* collision strengths */
	o10 = t10[IPLNCS-1];
	o21 = t21[IPLNCS-1];
	o20 = t20[IPLNCS-1];

	/*begin sanity checks */
	if( g010*g020 > 0. && g010 != g020 )
	{
		fprintf( ioQQQ, " level3: insane stat weights 020 010:%10.2e%10.2e %4.4s\n", 
		  g010, g020, chLab );
		insane();
		ShowMe();
		CloseUnits();
		puts( "[Stop in level3]" );
		exit(1);
	}

	if( g110*g121 > 0. && g110 != g121 )
	{
		fprintf( ioQQQ, " level3: insane stat weights 121 110:%10.2e%10.2e %4.4s\n", 
		  g110, g121, chLab );
		insane();
		ShowMe();
		CloseUnits();
		puts( "[Stop in level3]" );
		exit(1);
	}

	if( g221*g220 > 0. && g221 != g220 )
	{
		fprintf( ioQQQ, " level3: insane stat weights 220 221:%10.2e%10.2e %4.4s\n", 
		  g221, g220, chLab );
		insane();
		ShowMe();
		CloseUnits();
		puts( "[Stop in level3]" );
		exit(1);
	}

	/* both abundances must be same, equal abundance
	 * xIonFracs(nelem,i) is density of ith ionization stage (cm^-3) */
	if( t10[IPLNIONSTG-1]*t21[IPLNIONSTG-1] > 0. && t10[IPLNIONSTG-1] != 
	  t21[IPLNIONSTG-1] )
	{
		fprintf( ioQQQ, " level3: insane IonStg, given, array2=%10.2e\n", 
		  AbunxIon );
		insane();
		ShowMe();
		CloseUnits();
		puts( "[Stop in level3]" );
		exit(1);
	}

	if( t20[IPLNIONSTG-1]*t21[IPLNIONSTG-1] > 0. && t20[IPLNIONSTG-1] != 
	  t21[IPLNIONSTG-1] )
	{
		fprintf( ioQQQ, " level3: insane IonStg, given, array2=%10.2e\n", 
		  AbunxIon );
		insane();
		ShowMe();
		CloseUnits();
		puts( "[Stop in level3]" );
		exit(1);
	}

	if( t10[IPLNNELEM-1]*t21[IPLNNELEM-1] > 0. && t10[IPLNNELEM-1] != 
	  t21[IPLNNELEM-1] )
	{
		fprintf( ioQQQ, " level3: insane nelem, given, array2=%10.2e\n", 
		  AbunxIon );
		insane();
		ShowMe();
		CloseUnits();
		puts( "[Stop in level3]" );
		exit(1);
	}

	if( t20[IPLNNELEM-1]*t21[IPLNNELEM-1] > 0. && t20[IPLNNELEM-1] != 
	  t21[IPLNNELEM-1] )
	{
		fprintf( ioQQQ, " level3: insane nelem, given, array2=%10.2e\n", 
		  AbunxIon );
		insane();
		ShowMe();
		CloseUnits();
		puts( "[Stop in level3]" );
		exit(1);
	}

	if( (o10 <= 0. || o21 <= 0.) || o20 <= 0. )
	{
		fprintf( ioQQQ, " level3 called with non-positive collision strength, =%10.2e\n level3 called with non-positive collision strength, =%10.2e\n level3 called with non-positive collision strength, =%10.2e\n", 
		  o10, o21, o20 );
		fprintf( ioQQQ, " label=%10.10s\n", chLab );
		insane();
		ShowMe();
		CloseUnits();
		puts( "[Stop in level3]" );
		exit(1);
	}

	/*end sanity checks */

	/* net loss of line escape and destruction */
	a21 = t21[IPLNAUL-1]*(t21[IPLNESCP-1] + t21[IPLNDESP-1]);
	a10 = t10[IPLNAUL-1]*(t10[IPLNESCP-1] + t10[IPLNDESP-1]);
	a20 = t20[IPLNAUL-1]*(t20[IPLNESCP-1] + t20[IPLNDESP-1]);

	/* find energies of all transitions - one line could be a dummy
	 * also find boltzmann factors */
	if( a10 == 0. )
	{
		ener20 = t20[IPLNENRERG-1];
		ener21 = t21[IPLNENRERG-1];
		ener10 = ener20 - ener21;
		bolt12 = exp(-t21[IPLNBOLT-1]/phycon.te);
		bolt02 = exp(-t20[IPLNBOLT-1]/phycon.te);
		bolt01 = bolt02/bolt12;
		temp12 = t21[IPLNBOLT-1];
		temp02 = t20[IPLNBOLT-1];
		temp01 = temp02 - temp12;
	}

	else if( a21 == 0. )
	{
		ener10 = t10[IPLNENRERG-1];
		ener20 = t20[IPLNENRERG-1];
		ener21 = ener20 - ener10;
		bolt01 = exp(-t10[IPLNBOLT-1]/phycon.te);
		bolt02 = exp(-t20[IPLNBOLT-1]/phycon.te);
		bolt12 = bolt02/bolt01;
		temp02 = t20[IPLNBOLT-1];
		temp01 = t10[IPLNBOLT-1];
		temp12 = temp02 - temp01;
	}

	else if( a20 == 0. )
	{
		ener10 = t10[IPLNENRERG-1];
		ener21 = t21[IPLNENRERG-1];
		ener20 = ener21 + ener10;
		bolt01 = exp(-t10[IPLNBOLT-1]/phycon.te);
		bolt12 = exp(-t21[IPLNBOLT-1]/phycon.te);
		bolt02 = bolt01*bolt12;
		temp01 = t10[IPLNBOLT-1];
		temp12 = t21[IPLNBOLT-1];
		temp02 = temp01 + temp12;
	}

	else
	{
		/* all lines are ok */
		ener10 = t10[IPLNENRERG-1];
		ener20 = t20[IPLNENRERG-1];
		ener21 = t21[IPLNENRERG-1];
		bolt01 = exp(-t10[IPLNBOLT-1]/phycon.te);
		bolt12 = exp(-t21[IPLNBOLT-1]/phycon.te);
		bolt02 = bolt01*bolt12;
		temp02 = t20[IPLNBOLT-1];
		temp01 = t10[IPLNBOLT-1];
		temp12 = t21[IPLNBOLT-1];
	}

	/*begin sanity check
	 * check all energies positive */
	if( (ener10 < 0. || ener20 < 0.) || ener21 < 0. )
	{
		fprintf( ioQQQ, " level3 called with insane energy scale0 EN(10,21,20) =%10.2e%10.2e%10.2e\n", 
		  ener10, ener21, ener20 );
		fprintf( ioQQQ, " label=%10.10s\n", chLab );
		insane();
		ShowMe();
		CloseUnits();
		puts( "[Stop in level3]" );
		exit(1);
	}

	/* check if energy order is ok */
	if( ener10 > ener20 || ener21 > ener20 )
	{
		fprintf( ioQQQ, " level3 called with insane energy scale1 EN(10,21,20) =%10.2e%10.2e%10.2e\n", 
		  ener10, ener21, ener20 );
		fprintf( ioQQQ, " label=%10.10s\n", chLab );
		insane();
		ShowMe();
		CloseUnits();
		puts( "[Stop in level3]" );
		exit(1);
	}

	/* check if energy scale is ok */
	if( fabs((ener10+ener21)/ener20-1.) > 1e-4 )
	{
		fprintf( ioQQQ, " level3 called with insane energy scale2 EN(10,21,20) =%10.2e%10.2e%10.2e\n", 
		  ener10, ener21, ener20 );
		fprintf( ioQQQ, " label=%10.10s\n", chLab );
		insane();
		ShowMe();
		CloseUnits();
		puts( "[Stop in level3]" );
		exit(1);
	}

	/*end sanity check */

	pump01 = t10[IPLNPUMP-1];
	pump10 = pump01*g0/g1;
	pump12 = t21[IPLNPUMP-1];
	pump21 = pump12*g1/g2;
	pump02 = t20[IPLNPUMP-1];
	pump20 = pump02*g0/g2;

	/* cdsqte is 8.629E-6 / sqrte * eden */
	c01 = o10*bolt01*edsqte.cdsqte/g0;
	r01 = c01 + pump01;
	c10 = o10*edsqte.cdsqte/g1;
	r10 = c10 + a10 + pump10;
	c20 = o20*edsqte.cdsqte/g2;
	r20 = c20 + a20 + pump20;
	c02 = o20*bolt02*edsqte.cdsqte/g0;
	r02 = c02 + pump02;
	c12 = o21*bolt12*edsqte.cdsqte/g1;
	r12 = c12 + pump12;
	c21 = o21*edsqte.cdsqte/g2;
	r21 = c21 + a21 + pump21;

	alpha1 = (double)(AbunxIon)*(r01+r02)/(r10+r01+r02);
	alpha2 = (double)(AbunxIon)*(r01)/(r10+r12+r01);
	alpha = alpha1 - alpha2;

	/*  1( DBLE(r01+r02)/DBLE(r10+r01+r02) - DBLE(r01)/DBLE(r10+r12+r01) )
	 * beta is factor with n2 */
	beta = (r21 - r01)/(r10 + r12 + r01) + (r20 + r01 + r02)/(r10 + 
	  r01 + r02);

	if( alpha/MAX2(alpha1,alpha2) < 1e-11 )
	{
		/* this catches both negative and round off */
		p2 = 0.;
		alpha = 0.;
		Lev3Fail.nLev3Fail = 1;
	}

	else
	{
		p2 = alpha/beta;
	}
	PopLevls.PopLevels[2] = (float)p2;

	if( alpha < 0. || beta < 0. )
	{
		fprintf( ioQQQ, " level3: insane n2 pop alf, bet, p2=%10.2e%10.2e%10.2e %10.10s t=%10.2e\n", 
		  alpha, beta, p2, chLab, phycon.te );
		fprintf( ioQQQ, " gs are%5.1f%5.1f%5.1f\n", g0, g1, 
		  g2 );
		fprintf( ioQQQ, " Bolts are%10.2e%10.2e%10.2e\n", 
		  bolt01, bolt12, bolt02 );
		fprintf( ioQQQ, " As are%10.2e%10.2e%10.2e\n", a10, 
		  a21, a20 );
		fprintf( ioQQQ, " Energies are%10.2e%10.2e%10.2e\n", 
		  ener10, ener21, ener20 );
		fprintf( ioQQQ, " 2 terms, dif of alpha are%15.6e%15.6e\n", 
		  (r01 + r02)/(r10 + r01 + r02), r01/(r10 + r12 + r01) );
		ShowMe();
		CloseUnits();
		puts( "[Stop in level3]" );
		exit(1);
	}

	alpha = (double)(AbunxIon)*(r01+r02) - (double)(p2)*(r20+r01+r02);
	/* guard against roundoff - this should really have been zero
	 * >>chng 96 nov 14, protection against round-off to zero
	 * >>chng 96 dec 3, made r01, etc, double, and changed limit to 1e-9 */
	if( fabs(alpha)/(MAX2(AbunxIon*(r01+r02),p2*(r20+r01+r02))) < 1e-9 )
	{
		alpha = 0.;
		Lev3Fail.nLev3Fail = 2;
	}

	beta = r10 + r01 + r02;
	p1 = alpha/beta;
	PopLevls.PopLevels[1] = (float)p1;

	if( p1 < 0. )
	{
		if( p1 > -1e-37 )
		{
			/* slightly negative solution, probably just round-off, zero it */
			p1 = 0.;
			PopLevls.PopLevels[1] = (float)p1;
			Lev3Fail.nLev3Fail = 3;
		}

		else
		{
			/* very negative solution, better punt */
			fprintf( ioQQQ, " level3: insane n1 pop alf, bet, p1=%10.2e%10.2e%10.2e %10.10s%5ld\n", 
			  alpha, beta, p1, chLab, (long)(t10[IPLNWLANG-1]) );
			fprintf( ioQQQ, " local electron density and temperature were%10.2e%10.2e\n", 
			  phycon.eden, phycon.te );
			ShowMe();
			CloseUnits();
			puts( "[Stop in level3]" );
			exit(1);
		}
	}

	p0 = AbunxIon - p2 - p1;

	/* population of lowest level */
	PopLevls.PopLevels[0] = (float)p0;
	if( p0 <= 0. )
	{
		fprintf( ioQQQ, " level3: insane n0 pop p1, 2, abun=%10.2e%10.2e%10.2e \n", 
		  p1, p2, AbunxIon );
		ShowMe();
		CloseUnits();
		puts( "[Stop in level3]" );
		exit(1);
	}

	/* level populations for line opacities */
	t21[IPLNPOPL-1] = (float)p1;
	t10[IPLNPOPL-1] = (float)p0;
	t20[IPLNPOPL-1] = (float)p0;
	t21[IPLNPOPOPC-1] = (float)(p1 - p2*g1/g2);
	t10[IPLNPOPOPC-1] = (float)(p0 - p1*g0/g1);
	t20[IPLNPOPOPC-1] = (float)(p0 - p2*g0/g2);
	t21[IPLNPOPU-1] = (float)p2;
	t10[IPLNPOPU-1] = (float)p1;
	t20[IPLNPOPU-1] = (float)p2;

	/* line emission - net emission in line */
	t21[IPLNNPHOTS-1] = (float)(t21[IPLNAUL-1]*t21[IPLNESCP-1]*p2);
	t21[IPLNINTEN-1] = t21[IPLNNPHOTS-1]*t21[IPLNENRERG-1];
	t20[IPLNNPHOTS-1] = (float)(t20[IPLNAUL-1]*t20[IPLNESCP-1]*p2);
	t20[IPLNINTEN-1] = t20[IPLNNPHOTS-1]*t20[IPLNENRERG-1];
	t10[IPLNNPHOTS-1] = (float)(t10[IPLNAUL-1]*t10[IPLNESCP-1]*p1);
	t10[IPLNINTEN-1] = t10[IPLNNPHOTS-1]*t10[IPLNENRERG-1];

	/*  now add thess lines to ots field */
	AddOTSLin( t21[IPLNOTS-1] , (long)(t21[IPLNIPCONT-1]));
	AddOTSLin( t20[IPLNOTS-1] , (long)(t20[IPLNIPCONT-1]));
	AddOTSLin( t10[IPLNOTS-1] , (long)(t10[IPLNIPCONT-1]));

	/* total intensity used to divide line up - one may be 0 */
	TotInten = t21[IPLNINTEN-1] + t20[IPLNINTEN-1] + t10[IPLNINTEN-1];

	/* fraction that was collisionally excited */
	if( r12 > 0. )
	{
		t21[IPLNCOLOVTOT-1] = (float)(c12/r12);
	}
	else
	{
		t21[IPLNCOLOVTOT-1] = 0.;
	}

	if( r01 > 0. )
	{
		t10[IPLNCOLOVTOT-1] = (float)(c01/r01);
	}
	else
	{
		t10[IPLNCOLOVTOT-1] = 0.;
	}

	if( r02 > 0. )
	{
		t20[IPLNCOLOVTOT-1] = (float)(c02/r02);
	}
	else
	{
		t20[IPLNCOLOVTOT-1] = 0.;
	}

	/* heating or cooling due to each line */
	heat20 = p2*c20*ener20;
	cool02 = p0*c02*ener20;
	heat21 = p2*c21*ener21;
	cool12 = p1*c12*ener21;
	heat10 = p1*c10*ener10;
	cool01 = p0*c01*ener10;

	/* now get net heating or cooling */
	cnet02 = cool02 - heat20*t20[IPLNCOLOVTOT-1];
	hnet02 = heat20*(1. - t20[IPLNCOLOVTOT-1]);
	cnet12 = cool12 - heat21*t21[IPLNCOLOVTOT-1];
	hnet12 = heat21*(1. - t21[IPLNCOLOVTOT-1]);
	cnet01 = cool01 - heat10*t10[IPLNCOLOVTOT-1];
	hnet01 = heat10*(1. - t10[IPLNCOLOVTOT-1]);
	/* >>chng 96 nov 22, very dense cool models, roundoff error
	 *could cause [OI] 63 mic to be dominant heating, cooling, or
	 *just zero
	 * >>chng 96 dec 6, above change caused o iii 1666 cooling to
	 *be zeroed when important in a model in kirk's grid.  was at 1e-6,
	 * >>chng to 1e-7
	 * >>chng 96 dec 17, from 1e-7 to 1e-8, caused temp fail */
	if( fabs(cnet01/MAX2(1e-30,cool01)) < 1e-8 )
	{
		Lev3Fail.nLev3Fail = 4;
		cnet02 = 0.;
		hnet02 = 0.;
		cnet12 = 0.;
		hnet12 = 0.;
		cnet01 = 0.;
		hnet01 = 0.;
	}

	TotCool = cnet02 + cnet12 + cnet01;
	TotHeat = hnet02 + hnet12 + hnet01;

	if( TotInten > 0. )
	{
		t20[IPLNCOOL-1] = (float)(TotCool*(t20[IPLNINTEN-1]/TotInten));
		t21[IPLNCOOL-1] = (float)(TotCool*(t21[IPLNINTEN-1]/TotInten));
		t10[IPLNCOOL-1] = (float)(TotCool*(t10[IPLNINTEN-1]/TotInten));
		t20[IPLNHEAT-1] = (float)(TotHeat*(t20[IPLNINTEN-1]/TotInten));
		t21[IPLNHEAT-1] = (float)(TotHeat*(t21[IPLNINTEN-1]/TotInten));
		t10[IPLNHEAT-1] = (float)(TotHeat*(t10[IPLNINTEN-1]/TotInten));
	}
	else
	{
		Lev3Fail.nLev3Fail = 5;
		t20[IPLNCOOL-1] = 0.;
		t21[IPLNCOOL-1] = 0.;
		t10[IPLNCOOL-1] = 0.;
		t20[IPLNHEAT-1] = 0.;
		t21[IPLNHEAT-1] = 0.;
		t10[IPLNHEAT-1] = 0.;
	}

	/* fraction that produced lines */
	t21[IPLNAOVTOT-1] = (float)(t21[IPLNAUL-1]*t21[IPLNESCP-1]/r21);
	t10[IPLNAOVTOT-1] = (float)(t10[IPLNAUL-1]*t10[IPLNESCP-1]/r10);
	t20[IPLNAOVTOT-1] = (float)(t20[IPLNAUL-1]*t20[IPLNESCP-1]/r20);

	/* add cooling due to each line */
	coladd(chLab,(long)(t21[IPLNWLANG-1]),t21[IPLNCOOL-1]);
	coladd(chLab,(long)(t10[IPLNWLANG-1]),t10[IPLNCOOL-1]);
	coladd(chLab,(long)(t20[IPLNWLANG-1]),t20[IPLNCOOL-1]);

	/* derivative of cooling function
	 * dC/dT = Cooling * ( T(excitation)/T_e^2 - 1/(2T) )
	 * in following I assume that a 1-2 exciation will have the 0-2
	 * expnential in the dcdt term = NOT A TYPO */

	dcool.dCooldT += (float)(t10[IPLNCOOL-1]*(temp01*dcool.tsq1 - dcool.halfte) + 
	  (t20[IPLNCOOL-1] + t21[IPLNCOOL-1])*(temp02*dcool.tsq1 - dcool.halfte));
	/* two t20's above are not a typo!
	 * */

#	ifdef DEBUG_FUN
	fputs( " <->level3()\n", debug_fp );
#	endif
	return;
}

