/*PrintE82 - series of routines to mimic 1p, e8.2 fortran output */
#include "cddefines.h"
#include "printe82.h"
/***********************************************************
 * contains the following sets of routines to get around   *
 * the MS C++ compilers unusual exponential output.        *
 * PrintE103                                               *
 * PrintE102                                               *
 * PrintE93                                                *
 * PrintE92                                                *
 * PrintE82                                                *
 * PrintE81                                                *
 * PrintE71                                                *
 **********************************************************/

/**********************************************************/
void PrintE82( FILE* ioOut, double value )
{
	double frac , xlog , xfloor , tvalue;
	int iExp;
	if( value < 0 )
	{
		fprintf(ioOut,"********");
	}
	else if( value == 0 )
	{
		fprintf(ioOut,"0.00E+00");
	}
	else
	{
		/* round number off for 8.2 format (not needed since can't be negative) */
		tvalue = value;
		xlog = log10( tvalue );
		xfloor = floor(xlog);
		/* this is now the fractional part */
		frac = tvalue*pow(10.,-xfloor);
		/*this is the possibly signed exponential part */
		iExp = (int)xfloor;
		if( frac>9.9945 )
		{
			frac /= 10.;
			iExp += 1 ;
		}
		/* print the fractional part*/
		fprintf(ioOut,"%.2f",frac);
		/* E for exponent */
		fprintf(ioOut,"E");
		/* if positive throw a + sign*/
		if(iExp>=0 )
		{
			fprintf(ioOut,"+");
		}
		fprintf(ioOut,"%.2d",iExp);
	}

	return;
}
/*
 *==============================================================================
 */
void PrintE71( FILE* ioOut, double value )
{
	double frac , xlog , xfloor , tvalue;
	int iExp;
	if( value < 0 )
	{
		fprintf(ioOut,"*******");
	}
	else if( value == 0 )
	{
		fprintf(ioOut,"0.0E+00");
	}
	else
	{
		/* round number off for 8.2 format (not needed since can't be negative) */
		tvalue = value;
		xlog = log10( tvalue );
		xfloor = floor(xlog);
		/* this is now the fractional part */
		frac = tvalue*pow(10.,-xfloor);
		/*this is the possibly signed exponential part */
		iExp = (int)xfloor;
		if( frac>9.9945 )
		{
			frac /= 10.;
			iExp += 1 ;
		}
		/* print the fractional part*/
		fprintf(ioOut,"%.1f",frac);
		/* E for exponent */
		fprintf(ioOut,"E");
		/* if positive throw a + sign*/
		if(iExp>=0 )
		{
			fprintf(ioOut,"+");
		}
		fprintf(ioOut,"%.2d",iExp);
	}

	return;
}
/*
 *==============================================================================
 */void PrintE92( FILE* ioOut, double value )
{
	double frac , xlog , xfloor, tvalue;
	int iExp;
	if( value == 0 )
	{
		fprintf(ioOut," 0.00E+00");
	}
	else
	{
		/* round number off for 9.2 format, value may be negative */
		tvalue = fabs(value);
		xlog = log10( tvalue );
		xfloor = floor(xlog);
		/* this is now the fractional part */
		frac = tvalue*pow(10.,-xfloor);
		/*this is the possibly signed exponential part */
		iExp = (int)xfloor;
		/* take care of negative values*/
		if( value <0 ) frac = - frac;
		/* print the fractional part*/
		if( frac>9.9949 )
		{
			frac /= 10.;
			iExp += 1 ;
		}
		fprintf(ioOut,"%5.2f",frac);
		/* E for exponent */
		fprintf(ioOut,"E");
		/* if positive throw a + sign*/
		if(iExp>=0 )
		{
			fprintf(ioOut,"+");
		}
		fprintf(ioOut,"%.2d",iExp);
	}

	return;
} 
/*
 *==============================================================================
 */
void PrintE93( FILE* ioOut, double value )
{
	double frac , xlog , xfloor, tvalue;
	int iExp;
	if( value < 0 )
	{
		fprintf(ioOut,"*********");
	}
	else if( value == 0 )
	{
		fprintf(ioOut,"0.000E+00");
	}
	else
	{
		/* round number off for 9.3 format, neg numb not possilbe */
		tvalue = value;
		xlog = log10( tvalue );
		xfloor = floor(xlog);
		/* this is now the fractional part */
		frac = tvalue*pow(10.,-xfloor);
		/*this is the possibly signed exponential part */
		iExp = (int)xfloor;
		if( frac>9.99949 )
		{
			frac /= 10.;
			iExp += 1 ;
		}
		/* print the fractional part*/
		fprintf(ioOut,"%5.3f",frac);
		/* E for exponent */
		fprintf(ioOut,"E");
		/* if positive throw a + sign*/
		if(iExp>=0 )
		{
			fprintf(ioOut,"+");
		}
		fprintf(ioOut,"%.2d",iExp);
	}

	return;
}
/*
 *==============================================================================
 */
void PrintE103( FILE* ioOut, double value )
{
	double frac , xlog , xfloor, tvalue;
	int iExp;
	if( value == 0 )
	{
		fprintf(ioOut," 0.000E+00");
	}
	else
	{
		/* round number off for 10.3 format, value may be negative */
		/*tvalue = fabs(value)*1.000004;*/
		tvalue = fabs(value);
		xlog = log10( tvalue );
		xfloor = floor(xlog);
		/* this is now the fractional part */
		frac = tvalue*pow(10.,-xfloor);
		/*this is the possibly signed exponential part */
		iExp = (int)xfloor;
		/* take care of negative values*/
		if( value <0 ) frac = - frac;
		/* prevent 10.00e-3 type output */
		if( frac>9.9949 )
		{
			frac /= 10.;
			iExp += 1 ;
		}
		/* print the fractional part*/
		fprintf(ioOut,"%6.3f",frac);
		/* E for exponent */
		fprintf(ioOut,"E");
		/* if positive throw a + sign*/
		if(iExp>=0 )
		{
			fprintf(ioOut,"+");
		}
		fprintf(ioOut,"%.2d",iExp);
	}

	return;
} 
/*
 *==============================================================================
 */
void PrintE102( FILE* ioOut, double value )
{
	double frac , xlog , xfloor, tvalue;
	int iExp;
	if( value == 0 )
	{
		fprintf(ioOut,"  0.00E+00");
	}
	else
	{
		/* round number off for 10.2 format, value may be negative */
		tvalue = fabs(value);
		xlog = log10( tvalue );
		xfloor = floor(xlog);
		/* this is now the fractional part */
		frac = tvalue*pow(10.,-xfloor);
		/*this is the possibly signed exponential part */
		iExp = (int)xfloor;
		/* take care of negative values*/
		if( value <0 ) frac = - frac;
		if( frac>9.9949 )
		{
			frac /= 10.;
			iExp += 1 ;
		}
		/* print the fractional part*/
		fprintf(ioOut," %5.2f",frac);
		/* E for exponent */
		fprintf(ioOut,"E");
		/* if positive throw a + sign*/
		if(iExp>=0 )
		{
			fprintf(ioOut,"+");
		}
		fprintf(ioOut,"%.2d",iExp);
	}

	return;
} 
/*
 *==============================================================================
 */
void PrintE81( FILE* ioOut, double value )
{
	double frac , xlog , xfloor, tvalue;
	int iExp;
	if( value == 0 )
	{
		fprintf(ioOut," 0.0E+00");
	}
	else
	{
		/* round number off for 8.1 format, value may be negative */
		tvalue = fabs(value);
		xlog = log10( tvalue );
		xfloor = floor(xlog);
		/* this is now the fractional part */
		frac = tvalue*pow(10.,-xfloor);
		/*this is the possibly signed exponential part */
		iExp = (int)xfloor;
		/* take care of negative values*/
		if( value <0 ) frac = - frac;
		/* print the fractional part*/
		if( frac>9.949 )
		{
			frac /= 10.;
			iExp += 1 ;
		}
		fprintf(ioOut,"%4.1f",frac);
		/* E for exponent */
		fprintf(ioOut,"E");
		/* if positive throw a + sign*/
		if(iExp>=0 )
		{
			fprintf(ioOut,"+");
		}
		fprintf(ioOut,"%.2d",iExp);
	}

	return;
} 
