/*rdfile read grains parameters from a file. called by getgrain when grain number  >10 */
#include "cddefines.h"
#include "grainvar.h"
#include "lastdt.h"
#include "rdfile.h"

void rdfile( long int igr,
				char chLine[] )/* the name of the file */
{
	FILE * ioFILE ;
	long int  i,
		nfield ;
	float s1;
	char chCard[133];

#	ifdef DEBUG_FUN
	fputs( "<+>rdfile()\n", debug_fp );
#	endif

	/* read grains parameters from a file.  It is called when the grain number  >10
	 * The sublimation temperature is a new addition for c90.
	 * This subroutine added by (K Volk) 27 August 1993 */

	/* first try to open the file, 
	 * its name was read in with GetQuote in GetGrain, but only it quotes were entered */

	/* if first char is null then we know name was not properly entered */
	if( chLine[0]=='\0' )
	{
		/* big problem - we got here but double quotes were not picked up -
		 * not enough information to proceed */
		fprintf( ioQQQ , " rdfile got grain species that needed input file, but none specified.\n");
		fprintf( ioQQQ , " file name must occur between pair of double quotes.\n");
		CloseUnits();
		puts( "[Stop in rdfile]" );
		exit(1);
	}


	ioFILE = fopen( chLine , "r" );
	if( ioFILE == NULL )
	{
		fprintf( ioQQQ , "rdfile could not open grain opacity file named %s\n",chLine);
		CloseUnits();
		puts( "[Stop in rdfile]" );
		exit(1);
	}

	s1 = GrainVar.Tsublimat[igr-1];

	/* read first line */
	if( fgets( chCard , sizeof(chCard) , ioFILE ) == NULL )
	{
		fprintf( ioQQQ, " rdfile error getting first line\n" );
		CloseUnits();
		puts( "[Stop in rdfile]" );
		exit(1);
	}

	nfield = sscanf( chCard ,  " %10s%f %1li %12e %12e %7f",
	  chGrainVar.chDstLab[igr-1], 
	  &GrainVar.DustWorkFcn[igr-1], 
	  &lastdt.itype[igr-11], 
	  &lastdt.anumkv[igr-11], 
	  &GrainVar.qhnrat[igr-1], 
	  &GrainVar.Tsublimat[igr-1] );

	/* mak sure we got 6 values  */
	if( nfield != 6 )
	{
		fprintf( ioQQQ, " rdfile error parsing first line\n" );
		CloseUnits();
		puts( "[Stop in rdfile]" );
		exit(1);
	}

	/* The "old" dust input file does not have the sublimation temperature 
	 * so it is reset to the initial value if it is zero, or if the value is 
	 * absurdly small.
	 */
	if( GrainVar.Tsublimat[igr-1] <= 50. )
	{
		GrainVar.Tsublimat[igr-1] = s1;
	}

	/* read second line */
	if( fgets( chCard , sizeof(chCard) , ioFILE ) == NULL )
	{
		fprintf( ioQQQ, " rdfile error getting second line\n" );
		CloseUnits();
		puts( "[Stop in rdfile]" );
		exit(1);
	}

	nfield = sscanf( chCard ,"%9e %9e %9e  %9e %3li  %9e  %9e",
		&GrainVar.dustp[igr-1][0] ,
		&GrainVar.dustp[igr-1][1] ,
		&GrainVar.dustp[igr-1][2] ,
		&GrainVar.dustp[igr-1][3] ,
		&GrainVar.ndpts[igr-1], 
		&GrainVar.darea[igr-1], 
	   &GrainVar.dsize[igr-1]
		);
	/* make sure we got 7 values */
	if( nfield != 7 )
	{
		fprintf( ioQQQ, " rdfile error parsing second line\n" );
		CloseUnits();
		puts( "[Stop in rdfile]" );
		exit(1);
	}

	if( GrainVar.ndpts[igr-1] > LIMCRS )
		GrainVar.ndpts[igr-1] = LIMCRS;

	/* read third line */
	if( fgets( chCard , sizeof(chCard) , ioFILE ) == NULL )
	{
		fprintf( ioQQQ, " rdfile error getting third line\n" );
		CloseUnits();
		puts( "[Stop in rdfile]" );
		exit(1);
	}

/*		original fortran
      read(89,'(4(f5.1,1x,e12.4))', err=585,end=585) (dstelz(j,igr),
     1                              dstela(j,igr),j=1,4)
	  */
	nfield = sscanf( chCard ,"%5f %12e%5f %12e%5f %12e%5f %12e",
		&GrainVar.dstelz[igr-1][0], 
		&GrainVar.dstela[igr-1][0],
		&GrainVar.dstelz[igr-1][1], 
		&GrainVar.dstela[igr-1][1],
		&GrainVar.dstelz[igr-1][2], 
		&GrainVar.dstela[igr-1][2],
		&GrainVar.dstelz[igr-1][3], 
		&GrainVar.dstela[igr-1][3]
		);
	/* make sure we got 8 values */
	if( nfield != 8 )
	{
		fprintf( ioQQQ, " rdfile error parsing third line\n" );
		CloseUnits();
		puts( "[Stop in rdfile]" );
		exit(1);
	}

	for( i=0; i < GrainVar.ndpts[igr-1]; i++ )
	{
		/* read the data */
		if( fgets( chCard , sizeof(chCard) , ioFILE ) == NULL )
		{
			fprintf( ioQQQ, " rdfile error getting data, line=%li\n",i );
			CloseUnits();
			puts( "[Stop in rdfile]" );
			exit(1);
		}

		nfield = sscanf( chCard ,"%11e%11e%11e",
			&GrainVar.eev[igr-1][i], 
			&GrainVar.sab[igr-1][i], 
			&GrainVar.sse[igr-1][i]
			);
		/* make sure we got 8 values */
		if( nfield != 3 )
		{
			fprintf( ioQQQ, " rdfile error parsing data line, %li=\n",i );
			CloseUnits();
			puts( "[Stop in rdfile]" );
			exit(1);
		}
	}

	fclose( ioFILE );

#	ifdef DEBUG_FUN
	fputs( " <->rdfile()\n", debug_fp );
#	endif
	return;
}

