/*tav computes average of old and new optical depths for new scale at end of iter */
#include <string.h>
#include "cddefines.h"
#include "itercnt.h"
#include "taumin.h"
#include "sphere.h"
#include "double.h"
#include "prtmaser.h"
#include "chlinelbl.h"
#include "taulines.h"
#include "tav.h"

void tav(float t[], 
  double WeightNew)
{
	char chSave[11];

#	ifdef DEBUG_FUN
	fputs( "<+>tav()\n", debug_fp );
#	endif

	/* option to print masing lines, set with print maser */
	if( PrtMaser.lgPrtMaser && 
		( t[IPLNTAUTOT-1] < -0.01 || t[IPLNTAUIN-1] <  -0.01) )
	{
		strcpy( chSave, chLineLbl(t) );
		fprintf( ioQQQ, " Masing line:%10.10s t(in, out)=%10.2e%10.2e\n", 
		  chSave, t[IPLNTAUIN-1], t[IPLNTAUTOT-1] );
	}

	if( IterCnt.iter == 1 && (!sphere.lgStatic) )
	{
		/* first shot at model, set T(ipLnTauTot) to T(1)
		 * DoubleTau normally 1, set to 2 by DoubleTau command in order
		 * to simulate two-sided photoionization */
		t[IPLNTAUTOT-1] = t[IPLNTAUIN-1]*double_.DoubleTau;
		t[IPLNTAUIN-1] = (float)MIN2(tauminCom.taumin,t[IPLNTAUTOT-1]/2.);
	}
	else if( sphere.lgSphere && sphere.lgStatic )
	{
		/* static sphere, both sides interact */
		if( IterCnt.iter == 1 )
		{
			/* on first iteration TauIn was 0 for first zone */
			t[IPLNTAUTOT-1] = (float)(2.*t[IPLNTAUIN-1]);
		}
		else
		{
			t[IPLNTAUTOT-1] = (float)(t[IPLNTAUIN-1]*WeightNew + t[IPLNTAUTOT-1]*
			  (1. - WeightNew));
		}
		t[IPLNTAUIN-1] = (float)(t[IPLNTAUTOT-1]/2.);
	}
	else
	{
		/* take some sort of mean of old and new limiting optical depths */
		if( t[IPLNTAUIN-1] > 0. && t[IPLNTAUTOT-1] > 0. )
		{
			t[IPLNTAUTOT-1] = (float)pow(10.,log10(t[IPLNTAUTOT-1])*(1. - WeightNew) + 
				log10(t[IPLNTAUIN-1]*double_.DoubleTau)*WeightNew);
			t[IPLNTAUIN-1] = (float)MIN2(tauminCom.taumin,t[IPLNTAUTOT-1]/ 2.);
		}
		else
		{
			t[IPLNTAUTOT-1] = (float)((t[IPLNTAUIN-1]*double_.DoubleTau + 
			  t[IPLNTAUTOT-1])/2.);
			t[IPLNTAUIN-1] = (float)MIN2(tauminCom.taumin,t[IPLNTAUTOT-1]/2.);
		}
	}

	/* this is escape prob */
	t[IPLNESCP-1] = (float)(0.5*(1. + 1./MAX2(1.,t[IPLNTAUTOT-1])));

	/* this is fraction inward */
	t[IPLNINWD-1] = (float)MIN2(1.,1.5-t[IPLNESCP-1]);

	/* this is destruction probability
	 * >>chng 96 sep 4, was not here, needed to add since now taking
	 * mean of old and new dest prob */
	t[IPLNDESP-1] = 0.;

	/* line opacity */
	t[IPLNDTAU-1] = 0.;

	/* optical depth to the continuum source */
	t[IPLNTAUCON-1] = 0.;


#	ifdef DEBUG_FUN
	fputs( " <->tav()\n", debug_fp );
#	endif
	return;
}

