/*tfidle update some temperature dependent variables */
#include "cddefines.h"
#include "teinv.h"
#include "dcool.h"
#include "tepowers.h"
#include "edsqte.h"
#include "sqrne.h"
#include "densty.h"
#include "logte.h"
#include "phycon.h"
#include "h.h"
#include "elecden.h"
#include "showme.h"
#include "tfidle.h"

void tfidle(void)
{
	long int i;
	static double ttused = 0.;
	static double edused = 0.;
	static int lgZLogSet = FALSE;

#	ifdef DEBUG_FUN
	fputs( "<+>tfidle()\n", debug_fp );
#	endif

	/* check that eden not negative */
	if( phycon.eden <= 0. )
	{
		fprintf( ioQQQ, " I found a zero or negative electron density,%10.2e\n", 
		  phycon.eden );
		ShowMe();
		CloseUnits();
		puts( "[Stop in tfidle]" );
		exit(1);
	}

	/* check that temperature not negative */
	if( phycon.te <= 0. )
	{
		fprintf( ioQQQ, " I found a zero or negative electron temperature,%10.2e\n", 
		  phycon.te );
		ShowMe();
		CloseUnits();
		puts( "[Stop in tfidle]" );
		exit(1);
	}

	/* only time only, set up array of logs of charge squared */
	if( !lgZLogSet )
	{
		for( i=0; i<LIMELM; ++i )
		{
			/* this array is used to modify the log temperature array
			 * defined below, for hydrogenic species of charge i+1 */
			TePowers.sqlogz[i] = (float)log10( POW2(i+1.) );
		}
		lgZLogSet = TRUE;
	}

	if( phycon.te != ttused )
	{
		ttused = phycon.te;
		TePowers.sqrte = (float)sqrt(phycon.te);
		dcool.halfte = (float)(0.5/phycon.te);
		dcool.tsq1 = (float)(1./(POW2(phycon.te)));
		TePowers.te32 = phycon.te*TePowers.sqrte;
		TePowers.te70 = (float)pow(phycon.te,0.70);
		TePowers.te30 = (float)pow(phycon.te,0.30);
		TePowers.te20 = (float)pow(phycon.te,0.20);
		TePowers.te10 = (float)pow(phycon.te,0.10);
		TePowers.te05 = (float)pow(phycon.te,0.05);
		TePowers.te03 = (float)pow(phycon.te,0.03);
		TePowers.te02 = (float)pow(phycon.te,0.02);
		TePowers.te01 = (float)pow(phycon.te,0.01);
		TePowers.te005 = (float)pow(phycon.te,0.005);
		TePowers.te003 = (float)pow(phycon.te,0.003);
		TePowers.te001 = (float)pow(phycon.te,0.001);
		teinvCom.teinv = (float)(1./phycon.te);
		logte.alogte = (float)log10(phycon.te);
		logte.alogete = (float)log(phycon.te);
		TePowers.telogn[0] = logte.alogte;

		for( i=1; i < 7; i++ )
		{
			TePowers.telogn[i] = TePowers.telogn[i-1]*TePowers.telogn[0];
		}
	}

	/* term with hi added June 4, 93, to account for warm pdr */
	edsqte.edensqte = (float)((phycon.eden + h.hi/1e4)/TePowers.sqrte);
	edsqte.cdsqte = (float)(edsqte.edensqte*8.629e-6);

	/* used for hydrogenic collisions */
	ElecDen.EdenHCorr = (float)(phycon.eden + h.hi*1.7e-4);
	fixit();/* as test set EdenHCorr to eden */
	ElecDen.EdenHCorr = (float)phycon.eden;


	/* phycon.ElecFrac is electron fraction, used for secondary ionization efficiency */
	phycon.ElecFrac = (float)(phycon.eden/densty.TotalNuclei);

	logte.alogne = (float)log10(phycon.eden);

	if( phycon.eden != edused )
	{
		edused = phycon.eden;
		sqrne.SqrtEden = (float)sqrt(phycon.eden);
	}


#	ifdef DEBUG_FUN
	fputs( " <->tfidle()\n", debug_fp );
#	endif
	return;
}

